#include <stdlib.h>
#include <string.h>
#include <misc.h>
#include "projet.h"
#include "projetx.m"

static void near prjctrl_initlst (WILD_LST *lst)
{
	lst->nbelm = 0;
	lst->lst = NULL;
}

/*
	Spare une chaine en une liste de mot et accumule dans lst.
	Si lst contenait dj quelques chose, il est libr.
*/
static void near prjctrl_readlst (WILD_LST *lst, const char *pt)
{
	if (lst->lst != NULL){
		tbstr_free (lst->lst,lst->nbelm);
		free (lst->lst);
	}
	lst->lst = (char**) calloc_err (100,sizeof(char*),1);
	lst->nbelm = str_str2tb(pt,lst->lst,100,'\0');
	lst->lst[lst->nbelm] = NULL;
}

/*
	Ajoute un wildcard dans une liste.
	Vrifie s'il n'est pas dj l.
*/
static void near prjctrl_addlist (WILD_LST *lst, const char *wild)
{
	if (lst->lst == NULL){
		lst->lst = (char**) calloc_err (100,sizeof(char*),1);
	}
	int i;
	for (i=0; i<lst->nbelm; i++){
		if (strcmp(lst->lst[i],wild)==0) break;
	}
	if (i == lst->nbelm){
		lst->lst[i++] = strdup_err(wild);
		lst->lst[i] = NULL;
		lst->nbelm = i;
	}
}
/*
	Lit les spcifications controlant le traitement d'un projet.
	Si pas de fichier de controle, utilise des valeurs par dfaut.

	Certaines valeurs obligatoires sont ajouts de toutes faon, quelques
	soit le contenu du fichier de configuration.
	Retourne -1 si erreur.

	ATTENTION: Certaines des rubriques du fichier de configuration ne
			   servent plus.
*/
/* #Spcification: archivage / exclusions
	Certains types de fichiers sont non originaux. Ils sont produits
	 partir d'autres fichiers (gnralement via la commande mak export).
	Il n'est donc pas ncessaire de les archiver.
	On retrouve la liste de ces fichiers dans le source
	adm/projet/prjctrl.c dans la fonction PRJCTRL::PRJCTRL
*/
PUBLIC PRJCTRL::PRJCTRL (USERINFO *user)
{
	const char *refpath = user->getombre();
	prjctrl_initlst (&explst);
	prjctrl_initlst (&expobj);
	prjctrl_readlst (&expobj,"*.obj *.lib");
	prjctrl_initlst (&hidlst);
	prjctrl_initlst (&expmsg);
	char path[MAXSIZ_PATH];
	if (file_findanc (refpath,"projet.cfg",path) != -1
		|| file_findanc(refpath,"project.cfg",path) != -1){
		FILE *fin = fopen_err (path,"r",1);
		if (fin != NULL){
			char buf[300];
			while (fgets_strip (buf,sizeof(buf),fin,'\\',';',NULL)!=NULL){
				char mot[30];
				char *pt = str_copyword(mot,buf);
				if (pt != NULL){
					if (strcmp(mot,"#ignore")==0){
						prjctrl_readlst (&hidlst,pt);
					}else if (strcmp(mot,"#exporte")==0){
						prjctrl_readlst (&explst,pt);
					}else if (strcmp(mot,"#exporteobj")==0){
						prjctrl_readlst (&expobj,pt);
					}else if (strcmp(mot,"#message")==0){
						prjctrl_readlst (&expmsg,pt);
					}else{
						xconf_error (MSG_U(E_CONFIG
							,"Fichier de configuration invalide\n%s\n")
							,buf);
					}
				}
			}
			fclose (fin);
		}
	}else{
		// error_showdef (MSGERR_NOCONFIG,refpath);
	}
	static char *defval[]={
		"*.na*","*.i","*.p","*.m","names.mk","names2.mk","*.ob*","*.lib",
		"tlib.inp","modif.log","*.com","*.exe",
		"*.bak","*.n","*.old","makefile.dep","*.sym",
		"vista.log","vista.taf","errors","*.er?","*.pm","*.bld",
		"*.dbg","*.asv","*.$$$","*.map","*.sav","core",
		"projet.cfg","version.dat","makefile.dat","makefile.del",
		"revision.dat","change.dat","integre.dat","logfile.log",
		"state.rst"
	};
	for (unsigned i=0; i < NB_ELM(defval); i++){
		prjctrl_addlist(&hidlst,defval[i]);
	}
}

/*
	Libre les mots d'une liste
*/
static void near prjctrl_freelst (WILD_LST *lst)
{
	tbstr_free (lst->lst,lst->nbelm);
	free (lst->lst);
	lst->lst = NULL;
	lst->nbelm = 0;
}

/*
	Libre information de controle sur transaction d'un projet
*/
PUBLIC PRJCTRL::~PRJCTRL ()
{
	prjctrl_freelst (&explst);
	prjctrl_freelst (&expobj);
	prjctrl_freelst (&hidlst);
	prjctrl_freelst (&expmsg);
}

static char **tbstr;

static int fct(
	const FILEINFO *info,
	const char *,
	const char *,
	int nbf)
{
	int ok = 0;
	if (info->name[0] != '.'){
		char name[MAXSIZ_NAME];
		strcpy (name,info->name);
#ifdef MSDOS
		strlwr (name);
#endif
		if (strcmp(name,"h")!= 0
			&& strcmp(name,"doc")!=0
			&& name[strlen(name)-1] != '%'){	// backup de textedit
			if (info->type == FILETYPE_DIR) path_addback (name,name);
			tbstr[nbf] = strdup_err (name);
			ok = 1;
		}
	}
	return (ok);
}


/*
	Extrait la liste des sources d'un rpertoires.
	Obtient la liste des fichiers et des sous-rpertoires.
*/
/* #Spcification: archivage / exclusions
	Les sous-rpertoire h et doc ne sont jamais archivs. Leur contenu
	est non original. Il peut tre recontruit gnralement via les
	commandes "mak export" et "mak doc".
*/
PUBLIC int PRJCTRL::dirsource (
	const char *path,
	const char *wild,
	char *tb[],
	int )	// maxtb
{
	static const char *tbsel[]={NULL,NULL};
	tbstr = tb;
	tbsel[0] = wild;
	int nb = dir_gen (path,tbsel,(const char**)hidlst.lst,0,fct,0);
	tbstr_sort (tb,nb);
	return nb;
}

/*
	Dtermine si un nom obtenue via dirsource() est un directory.
	Retourne != 0 si oui.
*/
PUBLIC int PRJCTRL::isdir (const char *fname)
{
	const char *pt = fname + strlen(fname) -1;
	return path_issep(*pt);
}

