#include <stdlib.h>
#include <string.h>
#include <misc.h>
#include "projet.h"

/*
	Controle l'affichage des tapes de traitement excut par projet.
	Ce systme permet d'afficher ce que ralise projet au fur et  mesure.

	En plus d'afficher, le systme mmorise les messages. Il est alors
	possible de revenir en arrire.

	De plus, un fichier permet de retracer le traitement effectu
	durant une session.

	Evidemment, ce que vous venez de lire est du Vaporware. D'ailleurs
	une version de ce module est en chantier pour WINDOWS NT. Elle devrait
	tre disponible sur IBM370, AS400, VAX, HP ainsi que VIC-20
	et NINTENDO.
*/
PUBLIC WINDOW_LOG::WINDOW_LOG (
	const char *fname,		// Enregistre dans le fichier fname si != NULL
	int append)				// Ajoute ou cration du log.
	: PROJET_LOG (fname,append)
{
	curwin = -1;
	maxlin = 10;
	nbline = 0;
	tbline = (char*)malloc_err (10*80);
	if (fname == NULL){
		strcpy (fpath,":^)");
	}else{
		log_getpath (fname,fpath);
	}
}
PUBLIC WINDOW_LOG::~WINDOW_LOG()
{
	free (tbline);
	hide();
}
/*
	Dessine une ligne du log.
*/
PROTECTED void WINDOW_LOG::draw(int noline)
{
	#ifdef MISSING
		if (curwin != -1){
			int old = window_select (curwin);
			int posy = wininfo.lasty - pen_info.cell_topl
				- noline * pen_info.cell_height;
			draw_text (0,posy,1,1,"%s",tbline + noline*80);
			window_select (old);
		}
	#endif
}
/*
	Redessine toutes les lignes du log.
*/
PUBLIC void WINDOW_LOG::redraw()
{
	#ifdef MISSING
	if (curwin != -1){
		int old = window_select (curwin);
		window_clear (VDICOL_WHITE);
		for (int i=0; i<nbline; i++)draw(i);
		window_select (old);
	}
	#endif
}

/*
	Enregistre dans le window de log
*/
PROTECTED VIRTUAL void WINDOW_LOG::sendstr (const char *str)
{
	PROJET_LOG::sendstr (str);	// Enregistre dans le fichier
	while (*str != '\0'){
		char tmp[80];
		char *pt = tmp;
		/* #Spcification: log / window / tabulateur
			Les messages inscrit dans le log visuel peuvent contenir
			des tabulateur \t. Le tabulateur est standard  8 caractres.
		*/
		while (*str != '\0' && *str != '\n' && (int)(pt-tmp) < 75){
			if (*str == '\t'){
				str++;
				if ((pt-tmp)%8 == 0) *pt++ = ' ';
				while ((pt-tmp)%8 != 0) *pt++ = ' ';
			}else{
				*pt++ = *str++;
			}
		}
		*pt = '\0';
		if (*str == '\n') str++;
		if (nbline == maxlin){
			memmove (tbline,tbline + 5*80,5*80);
			nbline = 5;
			redraw();
		}
		strcpy (tbline+nbline*80,tmp);
		#ifdef MISSING
			draw(nbline);
		#endif
		nbline++;
	}
	#ifdef MISSING
		vdi_flush();
	#endif
}
#ifdef MISSING
static void redraw (void *data)
{
	WINDOW_LOG *log = (WINDOW_LOG*)data;
	log->redraw();
}
#endif
/*
	Affiche le log
*/
PUBLIC void WINDOW_LOG::show ()
{
	#ifdef MISSING
	if (curwin == -1){
		int basey = 2*pen_info.std_height;
		curwin = window_opengen (pen_info.std_width,basey
			,pen_info.std_width*78
			,basey+maxlin*pen_info.std_height + wininfo.box[WINBOX_SHADOWW].height
			,::redraw,this,1,0,0,fpath);
		window_setbox (curwin,WINBOX_SHADOWW,VDICOL_BLACK);
		redraw();
	}
	#endif
}
/*
	Efface le log. Le contenu n'est pas perdu.
*/
PUBLIC void WINDOW_LOG::hide ()
{
	#ifdef MISSING
	if (curwin != -1){
	#if	defined(MSDOS)
		printf ("------- Appuyez sur une touche -------\n");
	#else
		printf ("------- Appuyez sur un bouton --------\n");
	#endif
		event_wait();
		window_close (curwin);
		curwin = -1;
	}
	#endif
}


